<?php

namespace App\Http\Controllers\Admin;

use App\Models\User;
use App\Models\Tutor;
use App\Models\Course;
use App\Models\Category;
use App\Models\TutorCourse;
use Illuminate\Http\Request;
use App\Models\StudentCourse;
use App\Helper\fileManagerHelper;
use App\Http\Controllers\Controller;
use App\Http\Requests\CourseRequest;
use Illuminate\Support\Facades\Storage;

class CourseController extends Controller
{
    function __construct()
    {
        $this->middleware('permission:course-list', ['only' => ['index']]);
        $this->middleware('permission:course-create', ['only' => ['create', 'store']]);
        $this->middleware('permission:course-edit', ['only' => ['edit', 'update']]);
        $this->middleware('permission:course-delete', ['only' => ['destroy']]);
    }

    public function index()
    {
        $courses = Course::get();
        return view('admin.courses.index', compact('courses'));
    }

    public function create()
    {
        // $categories = Category::where('parent_id', null)->get();
        $categories = Category::get();
        $tutors = User::whereType('tutor')->get();
        return view('admin.courses.create', compact('categories', 'tutors'));
    }

    public function store(CourseRequest $request)
    {
        try {

            $result = fileManagerHelper::storefile($request->image, 'courses');
            if ($result['code'] == 500)
                return redirect()->route("admin.courses.create")->with(['message' => __('admin.upload file error'), 'type' => 'danger']);


            $show = $request->has('show') && $request->show == 1 ? 1 : 0;
            $status = $request->has('status') ? $request->status : 'close';

            $data = [
                'name' => ["en" => $request->name_en, "ar" => $request->name_ar],
                'start_date' => $request->start_date,
                'end_date' => $request->end_date,
                'price' => $request->price,
                'show' => $show,
                'status' => $status,
                'category_id' => $request->category_id,
                'image' => $result['filename'],
            ];

            $course = Course::create($data);

            $course_new = Course::find($course->id);
            $tutors = User::whereType('tutor')->find($request->tutors);
            $course_new->Tutors()->sync($tutors);
        } catch (\Exception $ex) {
            return redirect()->back()->with(['message' => __("admin.error_message"), 'error' => $ex->getMessage(), 'type' => 'danger']);
        }
        return redirect()->route('admin.courses.index')->with(['message' => __("admin.Has created successfully"), 'type' => 'success']);
    }

    public function edit($id)
    {
        $course = Course::with('Tutors')->find($id);
        if (!$course) {
            return redirect()->back()->with(['message' => __("admin.Item not exist"), 'type' => 'danger']);
        }
        $categories = Category::get();
        $tutors = User::whereType('tutor')->get();
        return view('admin.courses.edit', compact('course', 'categories', 'tutors'));
    }

    public function update(CourseRequest $request, $id)
    {
        $course = Course::find($id);
        if (!$course) {
            return redirect()->back()->with(['message' => __("admin.Item not exist"), 'type' => 'danger']);
        }

        try {
            $data = [];
            if ($request->hasFile('image')) {
                if (isset($course->image) && Storage::exists($course->image)) {
                    Storage::delete($course->image);
                }

                $result = fileManagerHelper::storefile($request->image, 'courses');
                if ($result['code'] == 500)
                    return redirect()->route("admin.courses.create")->with(['message' => __('admin.upload file error'),  'type' => 'danger']);

                $data['image'] = $result['filename'];
            }

            $show = $request->has('show') && $request->show == 1 ? 1 : 0;
            $status = $request->has('status') ? $request->status : 'close';

            $data['name'] = ["en" => $request->name_en, "ar" => $request->name_ar];
            $data['start_date'] = $request->start_date;
            $data['end_date'] = $request->end_date;
            $data['price'] = $request->price;
            $data['show'] = $show;
            $data['status'] = $status;
            $data['category_id'] = $request->category_id;
            // return $data;
            $course->update($data);
            // return $request->tutors;
            $tutors = User::whereType('tutor')->find($request->tutors);
            $course->Tutors()->sync($tutors);
        } catch (\Exception $ex) {
            return redirect()->back()->with(['message' => __("admin.error_message"), 'error' => $ex->getMessage(), 'type' => 'danger']);
        }
        return redirect()->route('admin.courses.index')->with(['message' => __("admin.Has updated successfully"), 'type' => 'success']);
    }

    public function change_status(Request $request, $id)
    {
        $course = Course::find($id);
        if (!$course) {
            return response()->json(['message' => __("admin.Item not exist"), 'type' => 'danger']);
        }
        try {
            $status = $request->has('status') && $request->status == 'open' ? 'open' : 'close';
            $data = [
                'status' => $status,
            ];
            $course->update($data);
        } catch (\Exception $ex) {
            return response()->json(['message' => __("admin.error_message"), 'error' => $ex->getMessage(), 'type' => 'danger']);
        }
        return response()->json(['message' => __("admin.Has updated successfully"), 'type' => 'success']);
    }
    public function change_show(Request $request, $id)
    {
        $course = Course::find($id);
        if (!$course) {
            return response()->json(['message' => __("admin.Item not exist"), 'type' => 'danger'], 500);
        }
        try {
            $show = $request->has('show') && $request->show == 1 ? 1 : 0;
            $data = [
                'show' => $show,
            ];
            $course->update($data);
        } catch (\Exception $ex) {
            return response()->json(['message' => __("admin.error_message"), 'error' => $ex->getMessage(), 'type' => 'danger', 'message' => $ex->getMessage()], 500);
        }
        return response()->json(['message' => __("admin.Has updated successfully"), 'type' => 'success']);
    }

    public function destroy($id)
    {
        $course = Course::find($id);
        if (!$course) {
            return redirect()->back()->with(['message' => __("admin.Item not exist"), 'type' => 'danger']);
        }

        try {

            $tutor = TutorCourse::where('course_id', $id)->count();
            if ($tutor > 0)
                return redirect()->back()->with(['message' => __("admin.cant delete courses because have tutors"), 'type' => 'danger']);

            $student = StudentCourse::where('course_id', $id)->count();
            if ($student > 0)
                return redirect()->back()->with(['message' => __("admin.cant delete courses because have students"), 'type' => 'danger']);


            $course->delete();
        } catch (\Exception $ex) {
            return redirect()->back()->with(['message' => $ex->getMessage(), 'type' => 'danger']);
        }
        return redirect()->route('admin.courses.index')->with(['message' => __("admin.Has deleted successfully"), 'type' => 'success']);
    }

    public function tutor($id)
    {
        $course = Course::with('tutors')->find($id);
        return view('admin.courses.tutors', compact('course'));
    }
}
