<?php

namespace App\Http\Controllers\Admin;

use App\Models\Product;
use Illuminate\Support\Str;
use App\Models\ProductImage;
use Illuminate\Http\Request;
use App\Helper\fileManagerHelper;
use App\Http\Controllers\Controller;
use App\Http\Requests\ProductImageRequest;
use App\Http\Requests\ProductRequest;
use Illuminate\Support\Facades\Storage;

class ProductController extends Controller
{
    function __construct()
    {
        $this->middleware('permission:product-index', ['only' => ['index']]);
        $this->middleware('permission:product-create', ['only' => ['create', 'store']]);
        $this->middleware('permission:product-edit', ['only' => ['edit', 'update']]);
        $this->middleware('permission:product-delete', ['only' => ['destroy']]);
    }

    public function index()
    {
        $products = Product::get();
        return view('admin.products.index', compact('products'));
    }

    public function create()
    {
        return view('admin.products.create');
    }

    public function store(ProductRequest $request)
    {
        try {

            $show = $request->has('show') && $request->show == 1 ? 1 : 0;
            $data = [
                'name' => ["en" => $request->name_en, "ar" => $request->name_ar],
                'slug' => ["en" => Str::slug($request->name_en), "ar" => Str::slug($request->name_ar)],
                'description' => ["en" => $request->desc_en, "ar" => $request->desc_ar],
                'price' => $request->price,
                'show' => $show,
            ];
            $product = Product::create($data);
        } catch (\Exception $ex) {
            return response()->json(['message' => __("admin.error_message"), 'error' => $ex->getMessage(), 'type' => 'danger'], 500);
        }
        return response()->json(['message' => __("admin.Has created successfully"), 'type' => 'success', 'data' => $product]);
    }

    public function store_image(ProductImageRequest $request)
    {
        try {

            $product = Product::find($request->product_id);
            if (!$product)
                return response()->json(['message' => __("admin.Item not exist"), 'type' => 'danger'], 500);

            // if (isset($product->image) && Storage::exists($product->image)) {
            //     Storage::delete($product->image);
            // }
            if (isset($product->image) && Storage::disk('public')->exists($product->image)) {
                Storage::delete('public/' . $product->image);
            }

            $result = fileManagerHelper::storefile($request->file, 'products');
            if ($result['code'] == 500)
                return response()->json(['message' => __('admin.upload file error'), 'type' => 'danger'], 500);

            $product->update(['image' => $result['filename']]);
        } catch (\Exception $ex) {
            return response()->json(['message' => __("admin.error_message"), 'error' => $ex->getMessage(), 'type' => 'danger'], 500);
        }
        return response()->json(['message' => __("admin.Has created successfully"), 'type' => 'success', 'data' => $product]);
    }

    public function edit($id)
    {
        $product = Product::with('Images')->find($id);
        if (!$product) {
            return redirect()->back()->with(['message' => __("admin.Item not exist"), 'type' => 'danger']);
        }
        // $product_images = ProductImage::where('product_id', $product->id);
        return view('admin.products.edit', compact('product'));
    }

    public function update(ProductRequest $request, $id)
    {
        $product = Product::find($id);
        if (!$product) {
            return redirect()->back()->with(['message' => __("admin.Item not exist"), 'type' => 'danger']);
        }

        try {
            $show = $request->has('show') && $request->show == 1 ? 1 : 0;
            $data = [
                'name' => ["en" => $request->name_en, "ar" => $request->name_ar],
                'description' => ["en" => $request->desc_en, "ar" => $request->desc_ar],
                'price' => $request->price,
                'show' => $show,
            ];

            $product->update($data);
        } catch (\Exception $ex) {
            return response()->json(['message' => __("admin.error_message"), 'error' => $ex->getMessage(), 'type' => 'danger'], 500);
        }
        return response()->json(['message' => __("admin.Has updated successfully"), 'type' => 'success', 'data' => $product]);
    }

    public function change_show(Request $request, $id)
    {
        // return $request->all();
        $product = Product::find($id);
        if (!$product) {
            return response()->json(['message' => __("admin.Item not exist"), 'type' => 'danger'], 500);
        }
        try {
            $show = $request->has('show') && $request->show == 1 ? 1 : 0;
            $data = [
                'show' => $show,
            ];
            $product->update($data);
        } catch (\Exception $ex) {
            return response()->json(['message' => __("admin.error_message"), 'type' => 'danger', 'error' => $ex->getMessage()], 500);
        }
        return response()->json(['message' => __("admin.Has updated successfully"), 'type' => 'success']);
    }

    public function destroy($id)
    {
        $product = Product::find($id);
        if (!$product) {
            return redirect()->back()->with(['message' => __("admin.Item not exist"), 'type' => 'danger']);
        }

        try {

            $images = ProductImage::where('product_id', $product->id)->get();
            foreach ($images as $img) {
                if (isset($img->path) && Storage::disk('public')->exists($img->path)) {
                    Storage::delete('public/' . $img->path);
                }
            }
            // $product->delete();
        } catch (\Exception $ex) {
            return redirect()->back()->with(['message' => __('admin.error_message'), 'error' => $ex->getMessage(), 'type' => 'danger']);
        }
        return redirect()->route('admin.products.index')->with(['message' => __("admin.Has deleted successfully"), 'type' => 'success']);
    }
}
